/* spsub.c */
/* subroutines for newstar spectral FITS files */
/* 93/06/15 generated by I.Yamamura */

#include <stdio.h>
#include <string.h>
#include <math.h>

#include "fits.h"
#include "macro.h"

char *IRASName(ra, dec, name)
/* position to IRAS name */
   double ra, dec;
   char name[];   /* array larger than 12 chars */
{

   int hh, mm, ms, dd, md;
   int i, f;

   ra /= 15.0;

   hh = (int)ra; ra = (ra - hh) * 60.0;
   mm = (int)ra; ra = (ra - mm) * 60.0;
   ms = ra / 6;

   if (dec < 0.0) {f = -1; dec = -dec;} else f = 1;
   dd = (int)dec; dec = (dec - dd) * 60.0;
   md = (int)dec;

   if (f == 1)
      sprintf(name, "%2d%2d%1d+%2d%2d", hh, mm, ms, dd, md);
   else
      sprintf(name, "%2d%2d%1d-%2d%2d", hh, mm, ms, dd, md);

   for (i = 0; i < 12 && name[i] != '\0'; i++)
      if (name[i] == ' ') name[i] = '0';

   return name;
} /* IRASName() */

char *PosStr(ra, dec, pos)
/* position string */
   double ra, dec;
   char **pos;
{

   int hh, mm, dd, md, sd;
   double ss;
   int i, f;

   (*pos) = (char *)calloc(14, sizeof(char));

   ra /= 15.0;

   hh = (int)ra; ra = (ra - hh) * 60.0;
   mm = (int)ra; ra = (ra - mm) * 60.0;
   ss = ra;

   if (dec < 0.0) {f = -1; dec = -dec;} else f = 1;
   dd = (int)dec; dec = (dec - dd) * 60.0;
   md = (int)dec; dec = (dec - md) * 60.0;
   sd = (int)dec;

   if (f == 1)
      sprintf((*pos), "%2d%2d%4.1f+%2d%2d%2d", hh, mm, ss, dd, md, sd);
   else
      sprintf((*pos), "%2d%2d%4.1f-%2d%2d%2d", hh, mm, ss, dd, md, sd);

   for (i = 0; i < 14 && (*pos)[i] != '\0'; i++)
      if ((*pos)[i] == ' ') (*pos)[i] = '0';

   return (*pos);
} /* PosStr() */

/* -------------------------------------------- */
char *FillZero(s)
/* exchange space in the strings by '0' */
   char *s;
{
   int i;

   i = 0;
   while(s[i] != '\0') {
      if (s[i] == ' ')
         s[i] = '0';
      i++;
   }
   return s;
} /* FillZero */

#define PI M_PI
#define OMEGA 4.926191815  /* 18h49m0 */
#define I 1.092576112      /* 62.60 deg */
#define L0 0.5759586532    /* 33.0 deg */

/* -------------------------------------------- */
void RaDecByDegree(ra, dec, alfa, delta)
/* get (alfa, delta) in [deg] */
   char *ra, *dec;
   double *alfa, *delta;
{
   int hh, dd, mm, ss;

   sscanf(FillZero(ra), "%2d%2d%3d", &hh, &mm, &ss);
   *alfa = (hh + mm / 60.0 + ss / 10.0 / 3600.0) * 15.0;
   sscanf(FillZero(dec), "%3d%2d%2d", &dd, &mm, &ss);
   *delta = (abs(dd) + mm / 60.0 + ss / 3600.0);
   if (dd < 0) 
      *delta = *delta * (-1.0);
} /* RaDecByDegree() */

/* -------------------------------------------- */
void AD2LB(alpha, delta, l, b)
/* (alpha, delta)(in degree) -> (l, b) */
   double alpha, delta;
   double *l, *b;
{
   double a, x, y;

   alpha = alpha / 180.0 * PI;
   delta = delta / 180.0 * PI;

   a = alpha - OMEGA;
   x = cos(delta) * cos(a);
   y = sin(delta) * sin(I) + cos(delta) * sin(a) * cos(I);
   *l = atan2(y, x) + L0;
   *b = asin(sin(delta) * cos(I) - cos(delta) * sin(a) * sin(I));

   *l = *l / PI * 180.0;
   *b = *b / PI * 180.0;
} /* AD2LB() */

void BugFix(spc, f)
/* correct bug(s) in fits file */
/* correct bug(s) in the routine 93/09/13 */
   long **spc;
   fits_header *f;
{
   int ig, ip;
   int idelt, ifqif;
   int size;
   double delt, fqif;

   size = f->naxis2 + f->pcount;

   for (ip = 0; ip < f->pcount; ip++) {
      if (strcmp((f->p[ip]).type, "CDELT2  ") == 0) idelt = ip;
      else if (strcmp((f->p[ip]).type, "FQIF1   ") == 0) ifqif = ip;
   }

   for (ig = 0; ig < f->gcount; ig++) {
      fqif = (*spc)[ig * size + ifqif] * f->p[ifqif].scal;
      delt = (*spc)[ig * size + idelt] * f->p[idelt].scal;
/* check and correct delt */
/*
      if (fqif < 0.0)   delt = fabs(delt);
      else              delt = -fabs(delt);
*/
/* 93/09/19 delt is always < 0 !? */
      delt = -fabs(delt);
      (*spc)[ig * size + idelt] = (long)(delt/f->p[idelt].scal);
   }
} /* BugFix() */

/* end of spsub.c */
