/* molco.c */
/* Initialize for CO molecule */
/*    programed by I.Yamamura 91/12/10 */
/*    colision rate routine changed to use intepolation of talbe */
/*       92/05/23 I.Yamamura */
/* Bug fix about Aij when (J-1 -> J)   92/08/22 */
/*    It's not Bug!!! recoverd   92/08/24 */
/* related to new made rtconfig.h 92/08/30 */
/*    since variables in rtvars.h are not used here, */
/*    this routine is independent to rtvars.h */
/* 92/11/15 Totally rewrited to use struct variables */
/* 93/10/28 definition of VMAX and JMAX are changed */
/*          they represent the number of level but not the maximum level */
/* 03/01/04 malloc.h -> stdlib.h */

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

#include "macro.h"
#include "const.h"
#include "funcs.h"
#include "matrix.h"

#include "mol.h"
#include "CO_E.inc"
#include "CO_He.inc"

#define i(v,j) ((v) * (JMAX) + (j))     /* array index */

#define VMAX  1               /* number of vibrational level */
#define JMAX  10              /* number of rotation level */
#define VJDIM (VMAX) * (JMAX)

#define MYU  0.1098           /* [debey] dipole moment */
#define B    3.845033/2.0     /* [cm-1] J=1 level energy */
#define EV   2143.2           /* [cm-1] v=1 level energy */

MOLECULE CO;

/* -------------------------------------------------------------- */
void CO_Init(CO)
   MOLECULE *CO;
{
   int v, j, vl, vu, jl, ju;

   strcpy(CO->name, "CO");

   CO->NV = VMAX;
   CO->NJ = JMAX;
   CO->LV = VJDIM;

/* memory allocation */
   CO->g =    (int *)calloc((unsigned)VJDIM, (unsigned)sizeof(int));
   CO->e =    (double *)calloc((unsigned)VJDIM, (unsigned)sizeof(double));
   CO->a =    d_alloc(VJDIM, VJDIM);
   CO->de =   d_alloc(VJDIM, VJDIM);
   CO->nyu0 = d_alloc(VJDIM, VJDIM);
   CO->ct =   d_alloc(VJDIM, VJDIM);

   CO->ng0 =  (double *)calloc((unsigned)VJDIM, (unsigned)sizeof(double));
   CO->ng =   (double *)calloc((unsigned)VJDIM, (unsigned)sizeof(double));

/* ----- enargy level */
   for (v = 0; v < VMAX; v++)
      for (j = 0; j < JMAX; j++)
         CO->e[i(v,j)] = EV * v + e_co[j];

/* ----- degeneracy */
   for (v = 0; v < VMAX; v++)
      for (j = 0; j < JMAX; j++)
         CO->g[i(v,j)] = 2 * j + 1;

/* ----- Av,j<->v,j-1 */
   for (vl = 0; vl < VMAX; vl++)
      for (vu = vl; vu < VMAX; vu++)
         for (jl = 0; jl < JMAX; jl++)
            for (ju = jl; ju < JMAX; ju++)

   if (vl == vu)
      if (ju == jl + 1)
         CO->a[i(vl,jl)][i(vu,ju)] = CO->a[i(vu,ju)][i(vl,jl)]
            = 3.137E-7 * ju/(double)(2*ju+1) * cub(CO->e[ju]-CO->e[jl]) * sqr(MYU);
      else
         CO->a[i(vl,jl)][i(vu,ju)] = CO->a[i(vu,ju)][i(vl,jl)] = 0.0;
   else if (vu == vl + 1)
      if (ju == jl + 1)
         CO->a[i(vl,jl)][i(vu,ju)] = CO->a[i(vu,ju)][i(vl,jl)] =
         CO->a[i(vu,jl)][i(vl,ju)] = CO->a[i(vl,ju)][i(vu,jl)]
            = 34.4 * ju / (double)(2*ju+1);
      else
         CO->a[i(vl,jl)][i(vu,ju)] = CO->a[i(vu,ju)][i(vl,jl)] =
         CO->a[i(vu,jl)][i(vl,ju)] = CO->a[i(vl,ju)][i(vu,jl)] = 0.0;
   else
      CO->a[i(vl,jl)][i(vu,ju)] = CO->a[i(vu,ju)][i(vl,jl)] =
      CO->a[i(vu,jl)][i(vl,ju)] = CO->a[i(vl,ju)][i(vu,jl)] = 0.0;

/* ----- energy difference and frequency */
   for (vl = 0; vl < VMAX; vl++)
      for (vu = 0; vu < VMAX; vu++)
         for (jl = 0; jl < JMAX; jl++)
            for (ju = 0; ju < JMAX; ju++) {
               CO->nyu0[i(vl,jl)][i(vu,ju)] = C*fabs(CO->e[i(vu,ju)]-CO->e[i(vl,jl)]);
               CO->de[i(vl,jl)][i(vu,ju)] = -(CO->e[i(vu,ju)]-CO->e[i(vl,jl)])/WK;
            }
} /* CO_Init() */

/* -------------------------------------------------------------- */
void CO_Stat(CO, nH2, abn, tp)
   MOLECULE *CO;
   double nH2, abn, tp;
{
#define QLIMIT 1.0E-15
   int v, j;
   double nsum, qj, f;
   static double eb, q, b;
   static double tsave = 0.0;

/* --- population --- */
   if (tp != tsave) {
      q = 0.0;
      j = 0;
      do {
         qj = 0.0;
         for (v = 0; v < VMAX; v++)
            qj = qj + (2*j+1) * exp(-(v*EV + B*j*(j+1))/WK/tp);
         q = q + qj;
         j++;
      } while (qj >= QLIMIT);
      tsave = tp;
   }
   CO->nmol = nH2 * abn;
   nsum = 0;
   for (v = 0; v < VMAX; v++) {
      for (j = 0; j < JMAX; j++) {
         CO->ng0[i(v,j)] = CO->ng[i(v,j)] =
               CO->nmol * exp(-CO->e[i(v,j)] / WK / tp) / q;
         nsum = nsum + CO->ng0[i(v,j)] * CO->g[i(v,j)];
      }
   }
/*   nmol = nsum;*/

   CO_Cik(CO, tp, nH2);

}  /* CO_Stat */

/* -------------------------------------------------------------- */
void CO_Cik(CO, tp, nH2)
   MOLECULE *CO;
   double tp, nH2;
/* --- cik --- */
/* This routine below is made by I.Yamamura by comparison with */
/* paper by Green et al. 91/09/05                              */
/* assume the molecule is excited/deexcited only between rotational levels */
/* 92/11/14 old formula is removed */
{
#define ACDIM 5
double spline();
   int v, it, j, k, ju, jl, dj;
   static int INIT = 0;
   static double ac[JMAX][JMAX][GCN];/**/
   double f, b;

/* new subroutine for calculating cik */
/* interplate talbe in Green and Chapman (same as above) */
/* 92/05/26 I.Yamamura */
/* ct[l][u] must be calculated from ct[u][l]. 92/06/09 */

   if (INIT == 0) {
      for (j = 0; j < JMAX; j++)
         for (k = 0; k < JMAX; k++) {
            if (j != k) {
               for (it = 0; it < GCN; it++)
                  if (GC[j][k][it] != 0.0) GC[j][k][it] = log(GC[j][k][it]*1.37);
               if (GC[j][k][0] != 0.0)
                  make_sp_table(9, GCT, GC[j][k], ac[j][k]);
               else
                  make_sp_table(8, &GCT[1], &GC[j][k][1], ac[j][k]);
            }
         }
      INIT = 1;
   }
   for (k = 0; k < JMAX; k++)
      for (j = k; j < JMAX; j++)
         if (j == k) CO->ct[j][k] = 0.0; else {
            if (tp > 100.0)
               CO->ct[j][k] = exp(GC[j][k][GCN-1]) * sqrt(tp/100.0) * nH2;
            else
               if (GC[j][k][0] != 0.0)
                  CO->ct[j][k] = exp(spline(tp, 9, GCT, GC[j][k], ac[j][k])) * nH2;
               else
                  CO->ct[j][k] = exp(spline(tp, 8, &GCT[1], &GC[j][k][1], ac[j][k])) * nH2;
               CO->ct[k][j] = CO->ct[j][k] * CO->g[j] / (double)CO->g[k] * exp(CO->de[k][j] / tp);
         }
} /* CO_Cik() */

/* end of molco.c */