/* convolv.c */
/* convolution by given beam patarn */
/* 93/08/23 I.Yamamura */
/* 03/01/04 malloc.h -> stdlib.h */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#include "const.h"
#include "macro.h"
#include "funcs.h"

#include "rt.h"

/* -------------------------------------------- */
void MakeBeam(HPBW, d, beam)
/* make beam pattern array (1 dimension) */
/* if HPBW == 0 then return flat beam 92/08/23 */
/* 93/08/23 not HPBW == 0 but HPBW <= 0 */
/* 93/08/23 renamed from InitEfc() with some changes offcource */
/* 93/08/23 eff = exp(-x^2 / (2 *sigma^2)) = (1 / 16)^((x / HPBW)^2) */
   double HPBW;         /* beamsize (arcsec) */
   AXIS *d;             /* dimension information (in arcsec) */
   double **beam;       /* output beam pattern */
{
   int iy, iymax;
   double e, r2, sum;
   double LN16 = log(16.0);

   (*beam) = (double *)calloc(d->n, sizeof(double));
   for (iy = 0; iy < d->n; iy++) (*beam)[iy] = 0.0;
   sum = 0.0;
   if (HPBW <= 0.0) {
      if (HPBW <= -100.0) {
         for (iy = 0; iy < d->n; iy++) {
            (*beam)[iy] = (iy + 0.5);
            sum += (iy + 0.5);
         }
      } else {
         for (iy = 0; iy < d->n; iy++) {
            (*beam)[iy] = 1.0;
            sum += 1.0;
         }
      }
   } else {
      iymax = (int)fabs(HPBW * 3.0 / d->delt);        /* where e = 1.5e-11 */
/*      if (iymax > d->n)
         fprintf(stderr, "MakeBeam : beam array is too short to cover the beam.\n");
*/
      for (iy = 0; iy <= iymax; iy++) {
         r2 = -sqr(iy * d->delt / HPBW);
         e = exp(LN16 * r2) * (iy + 0.5);
         if (iy < d->n) (*beam)[iy] = e;
         sum += e;
      }
   }
   for (iy = 0; iy < d->n; iy++)
      (*beam)[iy] /= sum;

} /* MakeBeam() */

/* -------------------------------------------- */
void MakeBeam2D(HPBWy, HPBWz, PA, db, beam)
/* make beam pattern map (2 dimension) */
/* if HPBW <= 0 then flat beam 93/08/23 */
/* 93/08/23 renamed from InitEfc2D() with changes */
/* input paramters : nb should be odd. db->ddelt should be given. */
   double HPBWy;        /* beamsize */
   double HPBWz;        /* beamsize */
   double PA;           /* position angle measured by counterclockwise (deg) */
   AXIS *db;            /* dimension information (y-axis) */
   double ***beam;      /* beam map */
{
   int iy, iz, iymax, izmax;
   double e, sum, theta;
   double cs, si;
   double Y, Z, Y2, Z2;
   double LN16 = log(16.0);

   if (db->delt == 0.0) {
      fprintf(stderr, "MakeBeam2D : d->ddelt == 0.0\n");
      exit(-1);
   }

   *beam = (double **)d_alloc(db->n, db->n);

   for (iz = 0; iz < db->n; iz++)
      for (iy = 0; iy < db->n; iy++)
         (*beam)[iz][iy] = 0.0;
   sum = 0.0;
   if (HPBWy <= 0.0 && HPBWz <= 0.0) {
      for (iz = 0; iz < db->n; iz++) {
         for (iy = 0; iy < db->n; iy++) {
            (*beam)[iz][iy] = 1.0;
            sum += 1.0;
         }
      }
   } else {
      theta = PA / 180.0 * PI;
      cs = cos(theta);  si = sin(theta);
      iymax = (int)fabs(HPBWy * 3.0 / db->delt);      /* where e = 1.5e-11 */
      izmax = (int)fabs(HPBWz * 3.0 / db->delt);
/*
      if (iymax > db->n / 2)
         fprintf(stderr, "MakeBeam2D : not enough beam array along y-axis.\n");
      if (izmax > db->n / 2)
         fprintf(stderr, "MakeBeam2D : not enough beam array along z-axis.\n");
*/

      for (iz = -izmax; iz <= izmax; iz++) {
         for (iy = -iymax; iy <= iymax; iy++) {
            Y =  iy * cs + iz * si;
            Z = -iy * si + iz * cs;
            if (HPBWy <= 0.0) Y2 = 0.0;
            else              Y2 = -sqr(Y * db->delt / HPBWy);
            if (HPBWz <= 0.0) Z2 = 0.0;
            else              Z2 = -sqr(Z * db->delt / HPBWz);
            e = exp(LN16 * (Y2 + Z2));
            if (0 <= iy + db->rpix && iy + db->rpix < db->n &&
                0 <= iz + db->rpix && iz + db->rpix < db->n)
                  (*beam)[iz + db->rpix][iy + db->rpix] = e;
            sum += e;
         }
      }
   }
   for (iz = 0; iz < db->n; iz++)
      for (iy = 0; iy < db->n; iy++)
         (*beam)[iz][iy] /= sum;

} /* MakeBeam2D() */

/* -------------------------------------------- */
void MakeBeamOffset(HPBW, d, offset, beam)
/* make beam pattern array with offset (1 dimension) */
/* 93/08/24 I.Yamamura */
   double HPBW;         /* beamsize (arcsec) */
   AXIS *d;             /* dimension information (in arcsec) */
   double offset;       /* offset (arcsec) */
   double **beam;       /* output beam pattern */
{
   int iy, iymax;
   double e, e1, e2, r, r2, ry, y, sum, theta;
   double LN16 = log(16.0);

   (*beam) = (double *)calloc(d->n, sizeof(double));
   for (iy = 0; iy < d->n; iy++) (*beam)[iy] = 0.0;
   sum = 0.0;
   if (HPBW <= 0.0) {
      for (iy = 0; iy < d->n; iy++) {
         (*beam)[iy] = 1.0;
         sum += 1.0;
      }
   } else {
      iymax = (int)fabs((offset + HPBW * 3.0) / d->delt);
/*
      if (iymax > d->n)
         fprintf(stderr, "MakeBeamOffset : beam array is too short to cover the beam.\n");
*/
      for (iy = 0; iy <= iymax; iy++) {
         y = iy * d->delt;
         r = sqrt((sqr(y) + sqr(offset)));
         r2 = -sqr(r / HPBW) * LN16;
         e1 = exp(r2);
         ry = -2.0 * offset * y / sqr(HPBW) * LN16;
         e2 = 0.0;
         for (theta = 0.0; theta <= PI; theta += PI / 20.0)
            e2 += exp(ry * cos(theta));
         e = 2.0 * e1 * e2 * r;
         if (iy < d->n) (*beam)[iy] = e;
         sum += e;
      }
   }
   for (iy = 0; iy < d->n; iy++)
      (*beam)[iy] /= sum;

} /* MakeBeamOffset() */

/* -------------------------------------------- */
void ImConv(em, d, f, beam, db, lim, cim)
/* make image map convolved by given beam */
/* 93/08/23 I.Yamamura */
   double ***em;        /* low image data */
   AXIS *d;             /* dimension of em */
   AXIS *f;             /* frequency data of em */
   double **beam;       /* beam data */
   AXIS *db;            /* dimension of beam */
   double lim;          /* approximate size of the beam */
   double ***cim;       /* convolved image data */
{
   int ic, iy, iz, iy2, iz2, idy, idz;
   int imax = (int)fabs(lim * 3.0 / d->delt);

   for (ic = 0; ic < f->n; ic++) {
      for (iz = 0; iz < d->n; iz++) {
         for (iy = 0; iy < d->n; iy++) {
            cim[ic][iz][iy] = 0.0;

   for (iz2 = iz - imax; iz2 <= iz + imax; iz2++) {
      idz = iz2 - iz + db->rpix;
      if ((0 < iz2 && iz2 < d->n) && (0 < idz && idz < db->n)) {
         for (iy2 = iy - imax; iy2 <= iy + imax; iy2++) {
            idy = iy2 - iy + db->rpix;
            if ((0 < iy2 && iy2 < d->n) && (0 < idy && idy < db->n)) {
               if (beam[idz][idy] > 0.0)
                  cim[ic][iz][iy] += em[ic][iz2][iy2] * beam[idz][idy];
            } /* if */
         } /* for (iy2) */
      } /* if */
   } /* for (iz2) */

         } /* for (iy) */
      } /* for (iz) */
   } /* for (ic) */

} /* ImConv() */

/* -------------------------------------------- */
void ImConvE(em, IY, IZ, d, f, beam, db, lim, cim)
/* make image map convolved by given beam */
/* do convolve at each point version */
/* 93/10/02 I.Yamamura */
   double *em;          /* LOS emergent intensity data */
   int IY, IZ;          /* position of the data */
   AXIS *d;             /* dimension of cim */
   AXIS *f;             /* frequency data of em */
   double **beam;       /* beam data */
   AXIS *db;            /* dimension of beam */
   double lim;          /* approximate size of the beam */
   double ***cim;       /* convolved image data */
{
   int ic, iy, iz, idy, idz;
   int imax = (int)fabs(lim * 3.0 / d->delt);
   static int FIRST = 0;

   if (FIRST == 0) {
      for (ic = 0; ic < f->n; ic++)
         for (iz = 0; iz < d->n; iz++)
            for (iy = 0; iy < d->n; iy++)
               cim[ic][iz][iy] = 0.0;
      FIRST = -1;
   }

   for (ic = 0; ic < f->n; ic++) {
      for (iz = IZ - imax; iz <= IZ + imax; iz++) {
         if (0 <= iz && iz < d->n) {
            idz = IZ - iz + db->rpix;
            for (iy = IY - imax; iy <= IY + imax; iy++) {
               if (0 <= iy && iy < d->n) {
                  idy = IY - iy + db->rpix;
                  if (beam[idz][idy] > 0.0)
                     cim[ic][iz][iy] += em[ic] * beam[idz][idy];
               } /* if */
            } /* for (iy) */
         } /* if */
      } /* for (iz) */
   } /* for (ic) */

} /* ImConvE() */

/* -------------------------------------------- */
void SpConv(em, d, f, beam, db, eff, spc)
/* make spectrum from 1-dimensional law data */
/* 93/08/24 I.Yamamura */
   double **em;         /* low intensity data */
   AXIS *d;             /* dimension of em */
   AXIS *f;             /* frequency data of em */
   double *beam;        /* beam data */
   AXIS *db;            /* dimension of beam */
   double eff;          /* main beam efficiency */
   double *spc;         /* convolved image data */
{
   int iv, iy;

   if (d->delt != db->delt) {
      fprintf(stderr, "SpConv : low data and beam mesh size inconsistent.\n");
      exit(-1);
   }

   if (eff <= 0.0) {
      for (iv = 0; iv < f->n; iv++) {
         spc[iv] = 0.0;
         for (iy = 0; iy < d->n; iy++) {
            spc[iv] += em[iv][iy];
         } /* for (iy) */
      } /* for (iv) */
   } else {
      for (iv = 0; iv < f->n; iv++) {
         spc[iv] = 0.0;
         for (iy = 0; iy < d->n; iy++) {
            spc[iv] += em[iv][iy] * beam[iy] * eff;
            } /* for (iy) */
      } /* for (iv) */
   }

} /* SpConv() */

/* -------------------------------------------- */
void SpConv2D(em, d, f, beam, db, eff, spc)
/* make spectrum from 2-dimensional law data */
/* 93/08/24 I.Yamamura */
   double ***em;        /* low intensity data */
   AXIS *d;             /* dimension of em */
   AXIS *f;             /* frequency data of em */
   double **beam;       /* beam data */
   AXIS *db;            /* dimension of beam */
   double eff;          /* main beam efficiency */
   double *spc;        /* convolved image data */
{
   int iv, iy, iz;

   if (d->delt != db->delt) {
      fprintf(stderr, "SpConv2D : low data and beam mesh size inconsistent.\n");
      exit(-1);
   }

   for (iv = 0; iv < f->n; iv++) {
      spc[iv] = 0.0;
      for (iz = 0; iz < d->n; iz++) {
         for (iy = 0; iy < d->n; iy++) {
            spc[iv] += em[iv][iz][iy] * beam[iz][iy] * eff;
         } /* for (iy) */
      } /* for (iz) */
   } /* for (iv) */

} /* SpConv2D() */

/* -------------------------------------------- */
void SpConv2DE(em, IY, IZ, f, beam, db, eff, spc)
/* make spectrum from 2-dimensional law data */
/* 93/08/24 I.Yamamura */
   double *em;          /* emergent intensity at IY, IZ */
   int IY, IZ;          /* position of em */
   AXIS *f;             /* frequency data of em */
   double **beam;       /* beam data */
   AXIS *db;            /* dimension of beam */
   double eff;          /* main beam efficiency */
   double *spc;         /* convolved image data */
{
   int iv, iy, iz;
   static int FIRST = 0;

   if (FIRST == 0) {
      for (iv = 0; iv < f->n; iv++) spc[iv] = 0.0;
      FIRST = -1;
   }

   for (iv = 0; iv < f->n; iv++) {
      spc[iv] += em[iv] * beam[IZ][IY] * eff;
   } /* for (iv) */

} /* SpConv2DE() */

/* -------------------------------------------- */
void FqBinning(em, f, spc, fo)
/* bininng of frequency axis */
/* 93/08/24 I.Yamamura */
   double *em;          /* high resolution data */
   AXIS *f;             /* frequency data of em */
   double *spc;        /* binned data */
   AXIS *fo;            /* binned frequency data */
{
   int iv, ic;
   double nyu;

   if (fo->delt == 0.0) {
      fprintf(stderr, "FqBinning : fo->delt == 0.\n");
      exit(-1);
   }

   for (ic = 0; ic < fo->n; ic++)
      spc[ic] = 0.0;

   for (iv = 0; iv < f->n; iv++) {
      nyu = (iv - f->rpix) * f->delt + f->rval;
      ic = nint((nyu - fo->rval) / fo->delt + fo->rpix);
      if (0 <= ic && ic < fo->n)
         spc[ic] += em[iv];
   } /* for (iv) */

   for (ic = 0; ic < fo->n; ic++)
      spc[ic] *= (f->delt / fo->delt);

} /* FqBinning() */

/* end of convolv.c */
