/* main.c */
/* 03/01/04 malloc.h -> stdlib.h */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#include "const.h"         /* physical constants */
#include "macro.h"         /* common macro functions */
#include "funcs.h"

#include "mol.h"
#include "model.h"
#include "rt.h"

/* these var-type are defined in model.h and vaiables are defined in model.c */
extern CONFIG cf;
extern MODEL  md;

/* ======================================================================= */
main()
{
   AXIS   d, db;     /* dimension */
   AXIS   f, fo;     /* frequency */
   LOS    cx;        /* line of sight */
   MOLECULE mol;
   double *beam;
   double **em;
   double *tb, *spc;

   int ix, iy, iv;
   int JU, JL;
   double y;
   double sec2cm;
   double nH2, tk, abn;
   double vsys;
   double v, nyu;

   int idx;
   double x0, xf;

   ReadParam();

   MolInit(md.C_MOLNM, &mol);

   JU = cf.JU;
   JL = JU - 1;

   cf.FWIDTH *= JU;
   cf.RESTFQ = mol.nyu0[JU][JL];

   d.n    = cf.SNPIX;
   d.rpix = cf.SRPIX;
   d.rval = cf.SRVAL;
   d.delt = cf.SDELT;

   f.n    = cf.FNPIX;
   f.rpix = cf.FRPIX;
   f.rval = Doppler(cf.VCENT, cf.RESTFQ);
   f.delt = cf.FWIDTH / (double)f.n;

   tb  = (double *)calloc(cf.FNPIX, sizeof(double));
   spc = (double *)calloc(cf.FNPIX, sizeof(double));
   em = (double **)d_alloc(cf.FNPIX, cf.SNPIX);

   db = d;
   MakeBeam(cf.HPBW, &db, &beam);/**/
/*   MakeBeamOffset(cf.HPBW, &db, 0.0, &beam);/**/

   InitLOS(cx, (cf.SNPIX * 2 - 1));

   sec2cm = tan(1.0 / 3600.0 / 180.0 * PI) * cf.DISTANCE * PCTOCM;
   vsys = md.C_VSYS;

   x0 = md.C_RI / 10.0;
   xf = exp(log(fabs(d.delt * sec2cm * cx.n / 2) / x0) / (cx.n / 2));

   for (ix = 0; ix < cx.n; ix++) {
      idx = ix - cx.n / 2;
      if (idx == 0)  cx.x[ix] = 0.0;
      else           cx.x[ix] = -1 * sgn(idx) * x0 * pow(xf, abs(idx) - 1.0);
   }

   for (iy = 0; iy < d.n; iy++) {
      y = ((iy - d.rpix) * d.delt + d.rval) * sec2cm;
      for (ix = 0; ix < cx.n; ix++) {
         cx.lc[ix] = Where(cx.x[ix], y, 0.0);
         if (cx.lc[ix] == LINE) {
            MolEnv(cx.x[ix], y, 0.0, &nH2, &tk, &abn,
               &cx.v[ix], &cx.turb[ix], &cx.dvdr[ix]);
            Pop(cx.x[ix], y, 0.0, nH2, abn, tk, cx.v[ix], cx.dvdr[ix],
               &mol, cf.LTE_M);
if (cx.x[ix] == 0.0) printf("%e %e\n", y, Tex(mol, JU, JL));
            cx.ngu[ix] = mol.ng[JU];
            cx.ngl[ix] = mol.ng[JL];
         } else if (cx.lc[ix] == CONT) {
            ContEnv(cx.x[ix], y, 0.0, &cx.ngu[ix], &cx.ngl[ix]);
         }
      }
      Trans(y, 0.0, JU, vsys, &mol, &cx, &f, tb, cf.LVG_M);
      for (iv = 0; iv < f.n; iv++)  em[iv][iy] = tb[iv];
   } /* iy */

   fo.n    = f.n;
   fo.rpix = f.rpix;
   fo.rval = cf.VCENT / 100.0;
   fo.delt = f.delt / cf.RESTFQ * C / 100.0;

   SpConv(em, &d, &fo, beam, &db, cf.EFF, spc);
   for (iv = 0; iv < fo.n; iv++) {
      v = (iv - fo.rpix) * fo.delt + fo.rval;
      nyu = Doppler(v, cf.RESTFQ);
      spc[iv] = ItoTb(nyu, spc[iv]);
   }
   WriteOneSp(cf.SPFLNM, spc, &fo, "VELO-LSR", TA);/**/

}

/* main.c */
